<?php

/**
 * Define the internationalization functionality
 *
 * Loads and defines the internationalization files for this plugin
 * so that it is ready for translation.
 *
 * @link       https://themify.me/
 * @since      1.0.0
 *
 * @package    Tbp
 * @subpackage Tbp/includes
 */

/**
 * Define the internationalization functionality.
 *
 * Loads and defines the internationalization files for this plugin
 * so that it is ready for translation.
 *
 * @since      1.0.0
 * @package    Tbp
 * @subpackage Tbp/includes
 * @author     Themify <themify@themify.me>
 */
class Tbp_Utils {

	public static $isLoop = false;
	public static $isActive = null;
	private static $excerpt_length = null;

	/**
	 * Get attachment image url by post_name.
	 * 
	 * @since 1.0.0
	 * @access public
	 * @param string $slug 
	 * @return string
	 */
	public static function get_attachment_url(string $slug): string {
		$attachment_id = self::get_post_type_query('ID', 'attachment', $slug, null);
		return $attachment_id ? wp_get_attachment_url($attachment_id) : '';
	}

	/**
	 * Custom query post type using $wpdb object.
	 * 
	 * @since 1.0.0
	 * @access public
	 * @return string|int|boolean
	 */
	public static function get_post_type_query(string $field,string $post_type, string $slug, string $post_status = 'publish') {
		global $wpdb;
		$post_status_query = is_null($post_status) ? '' : " AND post_status='" . $post_status . "'";
		$sql = $wpdb->prepare("SELECT " . $field . " FROM $wpdb->posts WHERE post_type='%s'" . $post_status_query . " AND post_name='%s' LIMIT 1", $post_type, $slug);
		return $wpdb->get_var($sql);
	}

	/**
	 * Get attachment image by post_name.
	 * 
	 * @since 1.0.0
	 * @access public
	 * @param string $slug 
	 * @param string $size 
	 * @param boolean $icon 
	 * @param string $attr 
	 * @return string
	 */
	public static function get_attachment_image(string $slug,string $size = 'thumbnail', $icon = false, $attr = ''):string {
		$attachment_id = self::get_post_type_query('ID', 'attachment', $slug, null);
		return $attachment_id ? wp_get_attachment_image($attachment_id, $size, $icon, $attr) : '';
	}

	public static function get_taxonomies() {
		$taxonomies = get_taxonomies(array('public' => true));
		$exclude_tax = array('post_format', 'product_shipping_class');

		// Exclude unnecessary taxonomies
		foreach ($exclude_tax as $tax) {
			unset($taxonomies[$tax]);
		}
		return array_map('get_taxonomy', $taxonomies);
	}

	public static function get_public_post_types(array $args = array()): array {
		$post_type_args = array(
			'show_in_nav_menus' => true,
		);

		if (!empty($args['post_type'])) {
			$post_type_args['name'] = $args['post_type'];
		}

		$_post_types = get_post_types($post_type_args, 'objects');

		$post_types = array();

		foreach ($_post_types as $post_type => $object) {
			$post_types[$post_type] = $object->labels->singular_name;
		}

		return $post_types;
	}

	private static function get_actual_viewing_post() {
		if (self::$isLoop === true) {
			return null;
		}
		global $post;
		$temp_post = $post;

		wp_reset_postdata();
		$return_post = $post;

		$post = $temp_post;

		return !empty($return_post) ? $return_post : null;
	}

	private static function get_actual_viewing_post_id():?int {
		$post = self::get_actual_viewing_post();
		if ($post) {
			return $post->ID;
		}

		return null;
	}

	public static function get_actual_query() {
		if (self::$isLoop === true) {
			return null;
		}
		static $res = null;
		if ($res !== null) {
			$res->rewind_posts();
			return $res;
		}
		$args = array(
			'order' => 'DESC',
			'orderby' => 'ID',
			'post_type' => 'any',
			'ptb_disable' => true,
			'posts_per_page' => 1,
			'no_found_rows' => true,
		);
		$cache = false;
		if (Themify_Builder::$frontedit_active === true || Tbp_Public::$isTemplatePage === true) {
			if (isset($_POST['tbp_post_id']) && is_numeric($_POST['tbp_post_id']) && get_post_status($_POST['tbp_post_id'])) {// id can be generated element id
				$id = (int) $_POST['tbp_post_id'];
			} elseif (isset($_POST['pageId'])) {
				$id = (int) $_POST['pageId'];
				$type = $_POST['type'];
				$isTerm = isset($_POST['terms'], $_POST['term_type']);
				$vals = array('post_type', 'order', 'orderby', 'offset');
				foreach ($vals as $v) {
					if (!empty($_POST[$v])) {
						$args[$v] = $_POST[$v];
					}
				}
				$vals = null;
				if (isset($args['offset'])) {
					$args['offset'] = (int) $args['offset'];
				}
				if ($isTerm === false && $type === 'category') {
					$args['cat'] = $id;
					$args['post_type'] = 'post';
				} elseif ($isTerm === false && $type === 'tag') {
					$args['tag'] = $id;
					$args['post_type'] = 'post';
				} elseif ($type === 'archive') {
					$args['post_type'] = $_POST['pageId'];
				} elseif ($type === '404') {
					$args['post_type'] = 'page';
					$args['p'] = $id;
				} elseif ($type === 'search') {
					$args['s'] = $_POST['pageId'];
				} elseif ($type === 'author') {
					$args['author'] = $id;
				} elseif (taxonomy_exists($type)) {
					if ($isTerm === true) {
						$args['ignore_sticky_posts'] = true;
						if ($_POST['term_type'] === 'post_slug') {
							if ($_POST['slug'] !== '') {
								$args['post__in'] = Themify_Builder_Model::parse_slug_to_ids(sanitize_text_field($_POST['slug']), $args['post_type']);
							}
						} else {
							Themify_Builder_Model::parseTermsQuery($args, sanitize_text_field($_POST['terms']), $type);
						}
					} else {
						$tax = get_taxonomy($type);
						if (!empty($tax)) {
							$args['tax_query'] = array(
								array(
									'taxonomy' => $type,
									'field' => 'id',
									'terms' => array($id)
								)
							);
							$args['post_type'] = $tax->object_type;
						}
						$tax = null;
					}
				} else {
					$args['p'] = $id;
					if (post_type_exists($type)) {
						$args['post_type'] = $type;
					}
				}
				if ($args['orderby'] === 'id') {
					$args['orderby'] = 'ID';
				}
				query_posts($args);
				global $wp_query;
				$res = $wp_query;
				Tbp_Public::set_rules();
				return $res;
			} elseif (Tbp_Public::$isTemplatePage === true) {
				$id = current(Tbp_Public::get_location());
				$cache = true;
			}
			if (isset($id)) {
				$post_type = get_post_type($id);
				if ($post_type === Tbp_Templates::SLUG) {
					$condition = Tbp_Templates::get_template_conditions($id);
					$post_type = self::get_post_type(Tbp_Templates::get_template_type($id), $condition[0]);
					$args['post_type'] = $post_type === 'any' ? 'post' : $post_type;
				} else {
					$args['p'] = $id;
				}
			} else {
				$args['post_type'] = 'post';
			}
		} else {
			$original_post = self::get_actual_viewing_post();
			if ($original_post) {
				$args['p'] = $original_post->ID;
				$args['post_type'] = $original_post->post_type;
			}
		}
		if ($args['orderby'] === 'id') {
			$args['orderby'] = 'ID';
		}
		$data = new WP_Query($args);
		if ($cache === true) {
			$res = $data;
		}
		return $data;
	}

	public static function get_wc_actual_query() {
		if (self::$isLoop === true) {
			return null;
		}
		static $res = null;
		if ($res !== null) {
			$res->rewind_posts();
			return $res;
		}
		$args = array(
			'post_type' => 'product',
			'ptb_disable' => true,
			'order' => 'DESC',
			'orderby' => 'ID',
			'posts_per_page' => 1,
			'no_found_rows' => true,
			'post_status' => current_user_can('edit_products') ? array('publish', 'draft', 'private') : 'publish',
			'ignore_sticky_posts' => true
		);
		if (Themify_Builder::$frontedit_active === true || Tbp_Public::$isTemplatePage === true) {
			$cache = Tbp_Public::$isTemplatePage === true;
			$args['tax_query'] = array(
				array(
					'taxonomy' => 'product_type',
					'field' => 'slug',
					'terms' => 'simple'
				),
				array(
					'taxonomy' => 'product_visibility',
					'field' => 'name',
					'operator' => 'NOT IN',
					'terms' => array('exclude-from-catalog', 'outofstock', 'exclude-from-search', 'featured')
				)
			);
			if (Themify_Builder::$frontedit_active === true) {

				if (isset($_POST['tbp_post_id']) && is_numeric($_POST['tbp_post_id']) && get_post_type($_POST['tbp_post_id']) === 'product') {
					$args['p'] = (int) $_POST['tbp_post_id'];
				} elseif (isset($_POST['pageId'])) {
					$id = (int) $_POST['pageId'];
					$type = $_POST['type'];
					if (!empty($_POST['aap'])) {
						if (!empty($_POST['terms'])) {
							Themify_Builder_Model::parseTermsQuery($args, sanitize_text_field($_POST['terms']), $type);
						}
					} elseif (taxonomy_exists($type)) {
						$args['tax_query'] = array(
							array(
								'taxonomy' => $type,
								'field' => 'id',
								'terms' => array($id)
							)
						);
					} else {
						$args['p'] = $id;
					}
					query_posts($args);
					global $wp_query;
					$res = $wp_query;
					Tbp_Public::set_rules();
					return $res;
				}
			}
		} else {
			$cache = false;
			$args['p'] = self::get_actual_viewing_post_id();
		}
		$data = new WP_Query($args);
		if ($cache === true) {
			$res = $data;
		}
		return $data;
	}

	public static function get_post_type(string $location, array $condition) {
		$general = isset($condition['general']) ? $condition['general'] : 'general';
		$query = isset($condition['query']) ? $condition['query'] : '';
		if ($location !== 'header' && $location !== 'footer') {
			$query = $general;
		} else {
			if ($general === 'general') {
				return 'any';
			}
			$location = $general;
		}
		if ($query === 'is_404' || $query === 'page' || $location === 'page' || $query === 'child_of' || ($query === 'is_front' && $location === 'single')) {
			return array('page');
		}
		if ($query === 'all' || $query === 'is_date' || $query === 'is_author' || $query === 'is_search') {
			return 'any';
		}
		if ($location === 'product_single' || $location === 'product_archive' || $query === 'product_cat' || $query === 'product_tag' || $query === 'product') {
			return array('product');
		}
		if ($query === 'post' || ($query === 'is_front' && $location === 'archive')) {
			return array('post');
		}
		if ($query === 'is_attachment') {
			return array('attachment');
		}
		if ($location === 'single' || $location === 'archive') {
			if ($location === 'archive' && strpos($query, 'all_') === 0) {
				return array(str_replace('all_', '', $query));
			}
			if (taxonomy_exists($query)) {
				$tax = get_taxonomy($query);
				return $tax->object_type;
			}
		}
		return array($query);
	}

	/**
	 * Manipulate post count in wp admin list table.
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public static function manipulate_views_count(string $what,array $views):array {
		global $wp_query;

		$total = $wp_query->post_count;
		$publish = $wp_query->post_count;

		$views['all'] = preg_replace('/\(.+\)/U', '(' . $total . ')', $views['all']);
		if (isset($views['publish'])) {
			$views['publish'] = preg_replace('/\(.+\)/U', '(' . $publish . ')', $views['publish']);
		}

		return $views;
	}

	public static function getProductImageUrl(array $params) {
		
	}

	public static function getLightBoxParams(array $args): string {

		$lightbox_settings = array();
		if (isset($args['lightbox_w']) && '' !== $args['lightbox_w']) {
			$lightbox_settings[] = $args['lightbox_w'] . $args['lightbox_w_unit'];
		}
		if (isset($args['lightbox_h']) && '' !== $args['lightbox_h']) {
			$lightbox_settings[] = $args['lightbox_h'] . $args['lightbox_h_unit'];
		}
		return implode('|', $lightbox_settings);
	}

	public static function getLinkParams(array $args, $link = ''): array {
		$attr = array();
		if (isset($args['no_follow']) && 'yes' === $args['no_follow']) {
			$attr['rel'] = 'nofollow';
		}
		if ($args['link'] === 'permalink') {
			$attr['href'] = $link !== '' ? $link : get_permalink();
		} elseif ($args['link'] === 'custom' && !empty($args['custom_link'])) {
			$attr['href'] = esc_url($args['custom_link']);
		} elseif ($args['link'] === 'media') {
			$attr['href'] = get_the_post_thumbnail_url();
			$attr['class'] = ' themify_lightbox';
		}
		if (isset($attr['href'])) {
			if ($args['open_link'] === 'newtab') {
				$attr['target'] = '_blank';
			} elseif ($args['open_link'] === 'lightbox') {
				$attr['class'] = ' themify_lightbox';
				if ((isset($args['lightbox_w']) && '' !== $args['lightbox_w']) || (isset($args['lightbox_h']) && '' !== $args['lightbox_h'])) {
					$attr['data-zoom-config'] = self::getLightBoxParams($args);
				}
			}
		}
		return $attr;
	}

	public static function getDateFormat(array $args): string {
		if (isset($args['format']) && 'def' !== $args['format']) {
			if ('custom' === $args['format']) {
				$format = !empty($args['custom']) ? $args['custom'] : '';
			} else {
				$format = $args['format'];
			}
		} else {
			$format = '';
		}
		return $format;
	}

	/**
	 * Get Modules, some modules can be disabled from Themify settings.
	 * @param string $slug 
	 * @param string $get
	 * @return array
	 */
	public static function get_module_settings($slug, $get = 'default') {//deprecated has been moved to js
		if (!isset(Themify_Builder_Model::$modules[$slug])) {
			static $isLoaded = array();
			if (!isset($isLoaded[$slug])) {
				Themify_Builder_Component_Module::load_modules($slug, true);
				$isLoaded[$slug] = Themify_Builder_Model::$modules[$slug] ? get_class(Themify_Builder_Model::$modules[$slug]) : false;
			}
			if (isset(Themify_Builder_Model::$modules[$slug])) {
				$module = Themify_Builder_Model::$modules[$slug];
				unset(Themify_Builder_Model::$modules[$slug]);
			} elseif ($isLoaded[$slug] !== false) {
				$module = new $isLoaded[$slug];
			}
		} else {
			$module = Themify_Builder_Model::$modules[$slug];
		}
		if (!isset($module)) {
			return array();
		}
		$data = $get === 'default' ? $module->get_live_default() : ($get === 'options' ? $module->get_options() : $module->get_styling());
		$module = null;
		return $data;
	}

	/**
	 * Disable PTB loop archive
	 * @return void
	 */
	public static function disable_ptb_loop() {
		if (function_exists('run_ptb')) {
			PTB_Public::get_instance()->disable_ptb(true);
		}
	}


	public static function is_multilingual(): bool {
		return defined('ICL_SITEPRESS_VERSION') || self::is_polylang();
	}

	/**
	 * Returns true if the website is using Polylang plugin
	 *
	 * @return bool
	 */
	public static function is_polylang(): bool {
		return defined('POLYLANG_VERSION');
	}

	/**
	 * Returns the current language code
	 *
	 * @since 1.0.0
	 *
	 * @return string the language code, e.g. "en"
	 */
	public static function get_current_language_code(): string {

		static $lng = null;
		if ($lng === null) {
			if (defined('ICL_LANGUAGE_CODE')) {
				$lng = ICL_LANGUAGE_CODE;
			} elseif (function_exists('qtrans_getLanguage')) {
				$lng = qtrans_getLanguage();
			}
			if (!$lng) {
				$lng = substr(get_bloginfo('language'), 0, 2);
			}
			$lng = strtolower(trim($lng));
		}
		return $lng;
	}

	/**
	 * Returns the current language code
	 *
	 * @since 1.0.0
	 *
	 * @return string the language code, e.g. "en"
	 */
	public static function get_default_language_code(): string {
		static $lng = null;
		if ($lng === null) {
			global $sitepress;
			if (isset($sitepress)) {
				$lng = $sitepress->get_default_language();
			} else if (function_exists('pll_default_language')) {
				$lng = pll_default_language();
			}
			if (empty($lng)) {
				$lng = substr(get_bloginfo('language'), 0, 2);
			}
			$lng = strtolower(trim($lng));
		}
		return $lng;
	}

	public static function get_all_languages(): array {
		static $languages = array();
		if (!empty($languages)) {
			return $languages;
		}
		if (defined('ICL_LANGUAGE_CODE')) {
			global $sitepress;
			$lng = self::get_current_language_code();
			foreach ($sitepress->get_active_languages() as $key => $l) {
				if ($lng === $key) {
					$languages[$key]['selected'] = true;
				}
				$languages[$key]['name'] = $l['native_name'];
			}
		} elseif (function_exists('qtrans_getLanguage')) {

			$languages = qtrans_getSortedLanguages();
		} else {

			$all_lang = self::get_default_language_code();
			$languages[$all_lang]['name'] = '';
			$languages[$all_lang]['selected'] = true;
		}

		return $languages;
	}

	public static function loadCssModules($slug, $url, $v = null) {
		static $isNew = null;
		if ($isNew === null) {
			$isNew = method_exists('Themify_Builder_Model', 'loadCssModules');
		}
		if ($v === null) {
			$v = TBP_VER;
		}
		if ($isNew === true) {
			Themify_Builder_Model::loadCssModules($slug, $url, $v);
		} elseif (function_exists('themify_enque_style')) {
			themify_enque_style($slug, $url, null, $v);
		} else {
			wp_enqueue_style($slug, $url, null, $v);
		}
	}

	/**
	 * Translate post or term ID to $language (default: current language)
	 *
	 * @return int|false
	 */
	public static function get_translated_object_id($object_id, $type) {
		if (defined('ICL_LANGUAGE_CODE')) {
			return apply_filters('wpml_object_id', $object_id, $type, false);
		} elseif (defined('POLYLANG_VERSION')) {
			return pll_get_post($object_id);
		}
	}

	/**
	 * Get the source post for a translation job in WPML plugin.
	 *
	 * @return string|null
	 */
	public static function wpml_get_trid_source_post($trid, $lang) {
		global $wpdb;

		return $wpdb->get_var(
				$wpdb->prepare(
					"SELECT element_id FROM {$wpdb->prefix}icl_translations WHERE trid='%d' AND language_code='%s' LIMIT 1",
					$trid,
					$lang
				)
		);
	}

	/**
	 * Gets a term $slus and returns its ID.
	 *
	 * With Polylang plugin, search is restricted to the default site language only.
	 *
	 * @return false|int
	 */
	public static function get_term_id_by_slug($slug, $taxonomy) {
		if (!taxonomy_exists($taxonomy)) {
			return false;
		}

		$args = array(
			'get' => 'all',
			'number' => 1,
			'taxonomy' => $taxonomy,
			'update_term_meta_cache' => false,
			'orderby' => 'none',
			'suppress_filter' => true,
			'slug' => (string) $slug,
		);

		if (self::is_polylang()) {
			$args['lang'] = Tbp_Utils::get_default_language_code();
		}

		$terms = get_terms($args);
		if (is_wp_error($terms) || empty($terms)) {
			return false;
		}

		return $terms[0]->term_id;
	}

	public static function get_post_id_by_slug($slug, $post_type) {
		$result = get_page_by_path($slug, OBJECT, $post_type);
		if ($result) {
			return $result->ID;
		}
	}

	/**
	 * Removes empty elements from multidimensional array
	 *
	 * @return array
	 */
	public static function array_filter_recursive(array $input): array {
		foreach ($input as &$value) {
			if (is_array($value)) {
				$value = self::array_filter_recursive($value);
			}
		}
		return array_filter($input);
	}



	/**
	 * Conditional Tag, check if the query is for any WC archive pages.
	 *
	 * @return bool
	 */
	public static function is_wc_archive(): bool {
		static $is = null;
		if ($is === null) {
			$is = themify_is_shop() || is_tax(get_object_taxonomies('product'));
		}

		return $is;
	}

	public static function has_access(string $key = 'tbp'): bool {
		static $is = null;
		if ($is === null) {
			$user = wp_get_current_user();
			if ($user->exists()) {
				$role = is_array($user->roles) ? $user->roles : array();
				$the_role = array_shift($role);
				if (empty($user->roles) || is_super_admin()) {
					$is = true;
				} else {
					$setting = 'setting-' . $key . '-' . $the_role;
					$is = themify_builder_get($setting, $setting, true) !== 'disable';
				}
			} else {
				$is = false;
			}
		}
		return $is;
	}

	/**
	 * Generate the_excerpt with consistent length trim & added ellipsis
	 * regardless of where the excerpt is coming from.
	 *
	 * @return string
	 */
	public static function get_excerpt(array $args = array()):string {
		$postId = isset($args['post_id']) ? $args['post_id'] : null;
		$post = get_post($postId);
		if (!$post) {
			return '';
		}
		$args['excerpt_length'] = !empty($args['excerpt_length']) ? $args['excerpt_length'] : 55;
		$args['ellipsis'] = isset($args['ellipsis']) && '1' === $args['ellipsis'] ? '' : ' [&hellip;]'; /* "1" : disable the ellipsis */
		$args['auto_excerpt'] = isset($args['auto_excerpt']) && '1' === $args['auto_excerpt'] ? false : true;

		if (has_excerpt($postId)) {
			/* if post has custom excerpt, "excerpt_length" and "excerpt_more" filters are not applied,
			 * so the excerpt text is trimmed manualy
			 */
			$value = wp_trim_words($post->post_excerpt, $args['excerpt_length'], $args['ellipsis']);
		} elseif ($args['auto_excerpt']) {
			self::$excerpt_length = $args['excerpt_length'];
			add_filter('excerpt_length', array(__CLASS__, 'excerpt_length'), 1000);
			if ($args['ellipsis'] === '') {
				add_filter('excerpt_more', array(__CLASS__, 'excerpt_more'), 1000);
			}

			$value = get_the_excerpt($postId);

			remove_filter('excerpt_length', array(__CLASS__, 'excerpt_length'), 1000);
			remove_filter('excerpt_more', array(__CLASS__, 'excerpt_more'), 1000);
			self::$excerpt_length = null;
		} else {
			$value = '';
		}
		if ($value !== '' && !empty($args['more'])) {
			$value .= $args['more'];
		}

		return $value;
	}

	public static function excerpt_length():?int {
		return self::$excerpt_length;
	}

	/**
	 * Disable ellipsis on post excerpt.
	 * Custom function in favor of __return_empty_string so it does not interfere with other filters.
	 *
	 * @return string
	 */
	public static function excerpt_more():string {
		return '';
	}

	/**
	 * Convert toggleable_fields associative array to numeric array 
	 */
	public static function convert_toggleable_fields(array &$array) {
        $old_more_link = isset( $array['more_l'] ) ? $array['more_l'] : false;
        unset( $array['more_l'] );

		if (!isset($array[0]) && !empty($array)) {
			$items = [];
			foreach ($array as $key => $vals) {
				$v = array(
					'id' => $key
				);
				if (isset($vals['on'])) {
					$v['on'] = $vals['on'];
				}
				if (isset($vals['val'])) {
					$v['val'] = $vals['val'];
				}

                /* backward compatibility with deprecated option */
                if ( $key === 'cont' && $old_more_link !== false && $old_more_link['on'] === '1' ) {
                    $v['val']['more_link'] = 'on';
                    if ( ! isset( $v['val']['more_text'] ) ) {
                        $v['val']['more_text'] = isset( $old_more_link['val']['link_text'] ) ? $old_more_link['val']['link_text'] : __('More &rarr;', 'tbp');
                    }
                }

				$items[] = $v;
			}
			$array = $items;
		}

	}

	
	/**
	 * Check if plugin(s) are active or not.
	 *
	 * @return bool
	 */
	public static function plugin_active($plugins):bool {
		foreach ((array) $plugins as $plugin) {
			if (!Themify_Builder_Model::is_plugin_active($plugin)) {
				return false;
			}
		}

		return true;
	}
}
